﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Drawing.Imaging;
using System.Globalization;
using System.IO;
using System.IO.Ports;
using System.Linq;
using System.Runtime.InteropServices;
using System.Threading;
using System.Text;
using System.Windows.Forms;
using System.Threading.Tasks;

namespace TSG2019.Demo
{
    public partial class Form1 : Form
    {
        Thread TSG2019CaptureHandle;
        Thread TSG2019RxThread;
        TSG2019.SignalGenerator TSG2019SG = new TSG2019.SignalGenerator();

        public Form1()
        {
            InitializeComponent();

            this.comboBox1.Items.AddRange(TSG2019SG.GetWorkingModeDesc());
            this.comboBox2.Items.AddRange(TSG2019SG.GetClockModeDesc());
            this.comboBox3.Items.AddRange(TSG2019SG.GetIQModeDesc());

            TSG2019CaptureHandle = new Thread(new ThreadStart(Run_TSG2019_Capture_Handle));
            TSG2019CaptureHandle.IsBackground = true;
            TSG2019CaptureHandle.Priority = ThreadPriority.Highest;
            TSG2019CaptureHandle.Start();
        }

        bool first = false;
        public delegate void UpdateTsgStatusInvoke(string sn);
        public void Run_TSG2019_Capture_Handle()
        {
            do
            {
                int res = TSG2019SG.GetDongleStatus();

                if (res == 1)
                {
                    if (TSG2019SG.GetUSBDongleHandle())
                    {
                        this.EndInvoke(this.BeginInvoke(new UpdateTsgStatusInvoke(UpdateTsgStatus), new Object[] { "dongle has connected!" + "  Serial Number:" + TSG2019SG.GetSerialNumber() }));
                    }
                }
                else if (res == 0)
                {
                    if (first == false)
                    {
                        first = true;

                        this.EndInvoke(this.BeginInvoke(new UpdateTsgStatusInvoke(UpdateTsgStatus), new Object[] { "dongle has disconnected!" }));

                        TSG2019SG.GetUSBDongleHandle();

                        
                    }
                }
                else
                {
                    first = false;
                }


            }
            while (true);
        }

        bool IsReceive = false;
        private void StartRxHandle()
        {
            IsReceive = true;

            TSG2019RxThread = new Thread(new ThreadStart(Run_TSG2019_Rx_Thread));
            TSG2019RxThread.IsBackground = true;
            TSG2019RxThread.Priority = ThreadPriority.Highest;
            TSG2019RxThread.Start();
        }

        public void Run_TSG2019_Rx_Thread()
        {
            do
            {
                bool res = Excute_TSG2019_Rx();

                if (res == false)
                {
                    IsReceive = false;
                }
            }
            while (IsReceive);
        }

        public delegate void UpdateTsgRxResultInvoke(string sn);
        public bool Excute_TSG2019_Rx()
        {
            bool xxx = false;

            double progress = 0;

            int res = TSG2019SG.ExcuteRxPacket(ref progress);

            string msg = string.Empty;

            if (res == 0)
            {
                if (progress == 0)
                {
                    msg = "command successfully";
                }
                else
                {
                    msg = (progress * 100) + "%";

                    if (progress < 1)
                    {
                        xxx = true;
                    }
                }
            }
            else
            {
                msg = "data error";
            }

            this.EndInvoke(this.BeginInvoke(new UpdateTsgRxResultInvoke(UpdateTsgRxStatus), new Object[] { msg }));

            return xxx;
        }

        private void UpdateTsgRxStatus(string msg)
        {
            this.lblProgress.Text = msg;
        }

        private void UpdateTsgStatus(string msg)
        {
            this.lblStaus.Text = msg;
            UpdateData();
        }

        bool comboNotWork = false;
        private void UpdateData()
        {
            try
            {

                comboNotWork = true;

                this.textBox1.Text = Convert.ToString(TSG2019SG.GetStartFrequency());
                this.textBox2.Text = Convert.ToString(TSG2019SG.GetStepFrequency());
                this.textBox3.Text = Convert.ToString(TSG2019SG.GetFrequencyPoints());

                ulong mainInterval = 0;
                ulong pulseWidth = 0;
                ulong pulseDelay = 0;
                byte pulseNumber = 0;

                TSG2019SG.GetTimerSetting(ref mainInterval, ref pulseWidth, ref pulseDelay, ref pulseNumber);

                this.textBox4.Text = Convert.ToString(mainInterval);
                this.textBox5.Text = Convert.ToString(pulseWidth);
                this.textBox6.Text = Convert.ToString(pulseDelay);
                this.textBox7.Text = Convert.ToString(pulseNumber);

                this.textBox8.Text = Convert.ToString(TSG2019SG.GetAmplitude());
                this.textBox9.Text = Convert.ToString(TSG2019SG.GetIQStepCount());

                this.checkBox1.Checked = TSG2019SG.RfIsOn();
                this.checkBox2.Checked = TSG2019SG.LowBandIsOn();

                this.comboBox1.SelectedIndex = TSG2019SG.GetWorkingModeCurrentIndex();
                this.comboBox2.SelectedIndex = TSG2019SG.GetClockModeCurrentIndex();
                this.comboBox3.SelectedIndex = TSG2019SG.GetIQModeCurrentIndex();

                comboNotWork = false;
            }
            catch
            {

            }
        }

        private void button3_Click(object sender, EventArgs e)
        {
            ulong startFrequency = 0;
            uint stepFrequency = 0;
            ushort points = 0;

            if (ulong.TryParse(this.textBox1.Text, out startFrequency) 
                && uint.TryParse(this.textBox2.Text, out stepFrequency)
                && ushort.TryParse(this.textBox3.Text, out points))
            {
                TSG2019SG.SetFrequency(startFrequency, stepFrequency, points);
                Excute_TSG2019_Rx();

                UpdateData();
            }
        }

        private void button4_Click(object sender, EventArgs e)
        {
            ulong mainInterval = 0;
            ulong pulseInterval = 0;
            ulong delayInterval = 0;
            byte pulseNumber = 0;

            if (ulong.TryParse(this.textBox4.Text, out mainInterval) 
                && ulong.TryParse(this.textBox5.Text, out pulseInterval) 
                && ulong.TryParse(this.textBox6.Text, out delayInterval) 
                && byte.TryParse(this.textBox7.Text, out pulseNumber))
            {
                TSG2019SG.SetTimer(mainInterval, pulseInterval, delayInterval, pulseNumber);
                Excute_TSG2019_Rx();
                UpdateData();
            }
        }

        private void textBox8_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (e.KeyChar == Convert.ToInt32(Keys.Enter))
            {
                double value = 0;

                if (double.TryParse(this.textBox8.Text, out value))
                {
                    TSG2019SG.SetAmplitude(value);
                    Excute_TSG2019_Rx();
                    UpdateData();
                }
            }
        }

        private void textBox9_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (e.KeyChar == Convert.ToInt32(Keys.Enter))
            {
                uint value = 0;

                if (uint.TryParse(this.textBox9.Text, out value))
                {
                    TSG2019SG.SetIqStepCount(value);
                    Excute_TSG2019_Rx();
                    UpdateData();
                }
            }
        }

        private void comboBox1_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (comboNotWork == false)
            {
                TSG2019SG.SetWorkingMode(Convert.ToByte(this.comboBox1.SelectedIndex));
                Excute_TSG2019_Rx();
                UpdateData();
            }
        }

        private void comboBox2_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (comboNotWork == false)
            {
                TSG2019SG.SetClockMode(Convert.ToByte(this.comboBox2.SelectedIndex));
                Excute_TSG2019_Rx();
                UpdateData();
            }
        }

        private void comboBox3_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (comboNotWork == false)
            {
                TSG2019SG.SetIQMode(Convert.ToByte(this.comboBox3.SelectedIndex));
                Excute_TSG2019_Rx();
                UpdateData();
            }
        }

        private void checkBox1_CheckedChanged(object sender, EventArgs e)
        {
            if (comboNotWork == false)
            {
                TSG2019SG.SetRfOn(this.checkBox1.Checked);
                Excute_TSG2019_Rx();
                UpdateData();
            }
        }

        private void checkBox2_CheckedChanged(object sender, EventArgs e)
        {
            if (comboNotWork == false)
            {
                TSG2019SG.SetLowBand(this.checkBox2.Checked);
                Excute_TSG2019_Rx();
                UpdateData();
            }
        }

        private void button2_Click(object sender, EventArgs e)
        {
            string FilePath = string.Empty;

            OpenFileDialog dlgOpenFileDialog = OpenFileDialog_Initial(System.Environment.GetFolderPath(System.Environment.SpecialFolder.MyDocuments), "Text File|*.txt");

            if (dlgOpenFileDialog.ShowDialog() == DialogResult.OK)
            {
                TSG2019SG.SendHoppingFile(dlgOpenFileDialog.FileName);
                StartRxHandle();
            }
        }

        private OpenFileDialog OpenFileDialog_Initial(string PathString, string FilterString)
        {
            OpenFileDialog dlgOpenFileDialog = new OpenFileDialog();

            dlgOpenFileDialog.InitialDirectory = PathString;

            dlgOpenFileDialog.Filter = FilterString;

            dlgOpenFileDialog.RestoreDirectory = true;

            dlgOpenFileDialog.FilterIndex = 1;

            return dlgOpenFileDialog;
        }

        private void button1_Click(object sender, EventArgs e)
        {
            string FilePath = string.Empty;

            OpenFileDialog dlgOpenFileDialog = OpenFileDialog_Initial(System.Environment.GetFolderPath(System.Environment.SpecialFolder.MyDocuments), "Text File|*.txt");

            if (dlgOpenFileDialog.ShowDialog() == DialogResult.OK)
            {
                TSG2019SG.SendIQRawDataFile(dlgOpenFileDialog.FileName);
                StartRxHandle();
            }
        }

        private void button5_Click(object sender, EventArgs e)
        {
            TSG2019SG.SetDefault();
            Excute_TSG2019_Rx();
            UpdateData();
        }


    }
}
